import { useState, useEffect, useRef, useImperativeHandle, forwardRef } from 'react'
import * as React from 'react';
export interface AudioPlayerProps {
  info: IMusic[]
  refs: any
  autoplay?: boolean
  children: any,
  curTime?: any,
  onPlaying?: (arg: boolean) => void
  onChange?: (arg: IMusic) => void
  onOperate?: (arg: IOperate) => void
  onTimeUpdate?: (currentTime: any, duration: any) => void
  onError?: (arg: any) => void
}
let rotateTimer: any = 0, angleNum: number = 0;
let audio: any;
const AudioPlayer: React.SFC<AudioPlayerProps> = (props: AudioPlayerProps, ref: any) => {

  const {
    info,
    refs,
    autoplay = false,
    children,
    onChange,
    onPlaying,
    onOperate,
    onError,
    curTime = 0,
    onTimeUpdate,
  } = props

  const [playing, setPlaying] = useState(autoplay)
  const [mouse, setMouse] = useState(false)
  const [currentMusic, setCurrentMusic] = useState<IMusic>(info[0])

  const audioRef = useRef(null);

  useEffect(() => {
    audio = audioRef && audioRef.current;
    if (audio) {
      audio.addEventListener('canplay', audioCanplay)
      audio.addEventListener('error', audioError)
      //设置初始音量
      if (refs.volumeProgress) {
        refs.volumeProgress.current.style.width = "50%";
      }
      audio.volume = 0.5
      audio.addEventListener('loadedmetadata', function () {
        audio.addEventListener('timeupdate', timeUpdate)
        audio.currentTime = curTime;
      });
    }
  }, [])

  useEffect(() => {
    if (playing) {
      setTimeout(() => {
        play()
      }, 1000);
    }
    return () => {
      clearInterval(rotateTimer)
      audio.removeEventListener('canplay', audioCanplay)
      audio.removeEventListener('error', audioError)
      audio.removeEventListener('timeupdate', timeUpdate)
      audio = null
    }
  }, [currentMusic.src])

  useEffect(() => {
    if (onPlaying && typeof onPlaying === 'function') {
      onPlaying(playing)
    }
    if (playing) {
      avatarTransform()
    } else {
      clearInterval(rotateTimer)
    }
  }, [playing])

  // 播放
  const play = () => {
    if (audio.paused && currentMusic.src) {
      const playPromise = audio.play();
      if (playPromise !== undefined) {
        playPromise
          .then(() => {
            setPlaying(true)
          })
          .catch(() => {
            setPlaying(false)
            audio.pause()
          });
      }
    } else {
      setPlaying(false)
      audio.pause()
    }
  }

  // 播放海报动画
  const avatarTransform = () => {
    if (refs.musicAvatar.current) {
      rotateTimer = setInterval(() => {
        angleNum = angleNum + 1
        refs.musicAvatar.current.style.transform = `rotate(${angleNum}deg)`;
      }, 70)
    }
  }

  // 监听播放总时间
  const audioCanplay = () => {
    const totalTime = parseInt(audio.duration);
    if (refs.totalTime.current) {
      refs.totalTime.current.innerHTML = getTime(totalTime)
    }
    if (refs.remainTime.current) {
      refs.remainTime.current.innerHTML = getTime(0)
    }
  }

  // 监听播放失败
  const audioError = () => {
    console.log("播放失败");
    if (onError && typeof onError === 'function') {
      onError('播放失败')
    }
  }

  // 监听音频时间
  const timeUpdate = () => {
    //设置播放进度条
    let playPer = audio.currentTime / audio.duration;
    if (refs.played.current) {
      refs.played.current.style.width = playPer * 100 + "%";
    }
    if (!audio.paused && onTimeUpdate && typeof onTimeUpdate === 'function') {
      onTimeUpdate(audio.currentTime, audio.duration)
    }
    //设置缓冲进度条
    let timeRages = audio.buffered;
    let bufferedTime = 0
    if (timeRages.length !== 0) {
      bufferedTime = timeRages.end(timeRages.length - 1);
    }
    let bufferedPer = bufferedTime / audio.duration;
    if (refs.buffered.current) {
      refs.buffered.current.style.width = bufferedPer * 100 + "%";
    }
    //设置剩余时间
    let remainTime = parseInt(audio.currentTime);

    if (refs.remainTime.current) {
      refs.remainTime.current.innerHTML = getTime(remainTime)
    }
    if (audio.ended) {
      // next()
      setPlaying(false)
      if (onOperate && typeof onOperate === 'function') {
        const v: IOperate = {
          option: 'end',
          music: currentMusic
        }
        return onOperate(v)
      }
    }
  }

  // 播放下一个
  const next = () => {
    angleNum = 0
    if (!currentMusic.src) {
      return
    }
    let current: number = 0
    info.forEach((v, i: number) => {
      if (v.src === currentMusic.src) {
        current = i
      }
    });
    if (current === info.length - 1) {
      if (audio.ended) {
        audio.pause()
        setPlaying(false)
      }
      if (onOperate && typeof onOperate === 'function') {
        const v: IOperate = {
          option: 'next',
          music: currentMusic
        }
        return onOperate(v)
      }
    }
    const data: IMusic = info[current < info.length - 1 ? current + 1 : 0]
    setCurrentMusic(data)
    if (onChange && typeof onChange === 'function') {
      onChange(data)
    }
  }

  // 播放上一个
  const last = () => {
    angleNum = 0
    if (!currentMusic.src) {
      return
    }
    let current: number = 0
    info.forEach((v, i: number) => {
      if (v.src === currentMusic.src) {
        current = i
      }
    });
    if (current === 0) {
      if (onOperate && typeof onOperate === 'function') {
        const v: IOperate = {
          option: 'prev',
          music: currentMusic
        }
        return onOperate(v)
      }
    }
    const data: IMusic = info[current > 0 ? current - 1 : info.length - 1]
    setCurrentMusic(data)
    if (onChange && typeof onChange === 'function') {
      onChange(data)
    }
  }

  // 快进快退15s
  const fastForward = (time: number) => {
    if (audio.currentTime !== 0) {
      audio.currentTime = audio.currentTime + time
      if (audio.paused) {
        audio.play()
        setPlaying(true)
      }
    }
  }

  // 获取时间
  const getTime = (musicTime: number) => {
    if (musicTime) {
      if (musicTime < 60) {
        return `00:${musicTime < 10 ? `0${musicTime}` : musicTime}`
      } else {
        const time: any = parseInt(`${musicTime / 60}`)
        return `${time < 10 ? `0${time}` : parseInt(time)}:${musicTime % 60 < 10 ? `0${musicTime % 60}` : musicTime % 60}`
      }
    } else {
      return `00:00`
    }
  }

  // 设置时间
  const setTime = (e: any) => {
    if (audio.currentTime !== 0 && refs.progress) {
      const pageX = !e.pageX ? e.touches[0].pageX : e.pageX
      if (pageX - refs.played.current.getBoundingClientRect().left <= refs.progress.current.offsetWidth && pageX - refs.played.current.getBoundingClientRect().left > 0) {
        const newWidth = (pageX - refs.played.current.getBoundingClientRect().left) / refs.progress.current.offsetWidth;
        refs.played.current.style.width = newWidth * 100 + "%";
        if (!mouse) {
          audio.currentTime = newWidth * audio.duration
        }
      }
      if (audio.paused) {
        audio.play()
        setPlaying(true)
      }
    }
  }

  // 设置音量
  const setVolume = (e: any) => {
    const volumeRate = (!e.pageX ? e.touches[0].pageX : e.pageX - refs.totalVolume.current.getBoundingClientRect().left) / refs.totalVolume.current.offsetWidth;
    if (volumeRate > 0.01 && volumeRate <= 1) {
      audio.volume = volumeRate
      if (refs.volumeProgress) {
        refs.volumeProgress.current.style.width = volumeRate * 100 + "%";
      }
    } else if (volumeRate <= 0.01) {
      audio.volume = 0
    } else {
      audio.volume = 1
    }
  }

  // 改变时间
  const changeTime = (e: any) => {
    if (audio.currentTime !== 0) {
      setTime(e)
    }
  }

  // 滑动时间
  const slideTime = (e: any) => {
    if (mouse && audio.currentTime !== 0) {
      setTime(e)
    }
  }

  // 改变音量
  const changeVolume = (e: any) => {
    if (audio.currentTime !== 0) {
      setVolume(e)
    }
  }

  // 滑动音量
  const slideVolume = (e: any) => {
    if (mouse && audio.currentTime !== 0) {
      setVolume(e)
    }
  }
  // 鼠标离开
  const mouseLeave = () => {
    setMouse(false);
  }
  // 鼠标按下
  const mouseDown = () => {
    setMouse(true);
  }
  // 播放指定音乐
  const playMusic = (id: string) => {
    const music: IMusic[] = info.filter(v => v.id === id)
    if (music.length > 0) {
      setPlaying(true)
      setCurrentMusic(music[0])
    } else {
      console.warn('没有此音乐')
    }
  }

  // 当前播放时间
  const onTime = () => {
    if (audio && audio.currentTime !== 0) {
      return audio.currentTime
    }
    return 0
  }

  // 当前播放状态
  const isPlay = () => {
    return playing || false
  }

  // 开放Api
  useImperativeHandle(ref, () => ({
    playMusic,
    play,
    next,
    last,
    mouseLeave,
    mouseDown,
    slideTime,
    changeTime,
    slideVolume,
    changeVolume,
    fastForward,
    onTime,
    isPlay,
  }))
  return (
    <div>
      {children}
      <audio src={currentMusic.src ? currentMusic.src : ""} ref={audioRef}></audio>
    </div>
  )

}

export default forwardRef(AudioPlayer)
