import * as React from 'react';
import { useState, useEffect, useRef, createRef, useImperativeHandle, forwardRef } from 'react'
import { Icon, message } from 'antd';
import AudioPlayer from '../../player'
import './style.css';

export interface IMusic {
  img?: string;
  id: string;
  artist?: string;
  name?: string;
  src: string;
}
export interface IOperate {
  option: string;
  music: IMusic;
}
export interface PlayProps {
  info: IMusic[]
  autoplay: boolean,
  currentTime?: any,
  onChange: (arg: IMusic) => void
  onOperate: (arg: IOperate) => void
  onTimeUpdate?: (currentTime: any, duration: any) => void
  onPlay?: (arg: boolean) => void
}

const WebAudio: React.SFC<PlayProps> = (props, ref) => {
  const {
    info,
    autoplay,
    onChange,
    onOperate,
    currentTime,
    onTimeUpdate,
    onPlay,
  } = props
  const [playing, setPlaying] = useState(false)
  const [music, setMusic] = useState<IMusic>()
  let musicRef: any = createRef();
  let refs: any = {
    musicAvatar: useRef(),
    progress: useRef(),
    buffered: useRef(),
    played: useRef(),
    totalTime: useRef(),
    remainTime: useRef(),
    totalVolume: useRef(),
    volumeProgress: useRef()
  }

  // 开放Api
  useImperativeHandle(ref, () => ({
    onPlay: () => {
      musicRef.current.play()
    },
    onTime: () => {
      return musicRef.current.onTime()
    },
    isPlay: () => {
      return musicRef.current.isPlay()
    }
  }))

  useEffect(() => {
    setMusic(info[0])
  }, [])


  const handleMusic = (method: string) => {
    musicRef.current[method]()
  }

  const onMusicEvents = (e:any, method: string) => {
    musicRef.current[method](e)
  }

  const hanleChange = (value: IMusic) => {
    setMusic(value)
    if (onChange && typeof onChange === 'function') {
      onChange(value)
    }
  }

  const hanleOption = (value: IOperate) => {
    if (onOperate && typeof onOperate === 'function') {
      onOperate(value)
    }
  }

  const hanlePlay = (value: boolean) => {
    setPlaying(value)
    if (onPlay && typeof onPlay === 'function') {
      onPlay(value)
    }
  }

  const hanleTimeUpdate = (currentTime: any, duration: any) => {
    if (onTimeUpdate && typeof onTimeUpdate === 'function') {
      onTimeUpdate(currentTime, duration)
    }
  }

  return (
    <AudioPlayer
      refs={refs}
      ref={musicRef}
      info={info}
      autoplay={autoplay}
      curTime={currentTime}
      onPlaying={(value: boolean) => hanlePlay(value)}
      onChange={(value: IMusic) => hanleChange(value)}
      onOperate={(value: IOperate) => hanleOption(value)}
      onError={error => message.error(error)}
      onTimeUpdate={(currentTime: any, duration: any) => hanleTimeUpdate(currentTime, duration)}
    >
      <div className='player'>
        <div className='music-player'>
          <div className='music-box'>
            <div className='picture'>
              <img src={music && music.img} ref={refs.musicAvatar} alt="图片丢失了" />
            </div>
            <div className='bar'>
              <div className={!playing ? 'demand-off' : 'demand-on'}></div>
            </div>
          </div>
        </div>
        <div className='music-control'>
          <div className='music-info'>
            <div className='time'>
              <span className='remain-time' ref={refs.remainTime}>00:00</span>
              <span>/</span>
              <span className='total-time' ref={refs.totalTime}>00:00</span>
            </div>
            <div className='music-name'>
              {/*music && `${music.name}-${music.artist}`*/}
            </div>
            <div className='progress-wrapper' ref={refs.progress}
              onTouchMove={(e) => onMusicEvents(e, 'changeTime')}
              onTouchStart={(e) => onMusicEvents(e, 'changeTime')}
              onClick={(e) => onMusicEvents(e, 'changeTime')}
              onMouseDown={(e) => onMusicEvents(e, 'mouseDown')}
              onMouseMove={(e) => onMusicEvents(e, 'slideTime')}
              onMouseUp={(e) => onMusicEvents(e, 'mouseLeave')}
              onMouseLeave={(e) => onMusicEvents(e, 'mouseLeave')}
            >

              <div className='progress'>
                <div className='progress-buffered' ref={refs.buffered}></div>
                <div className='progress-played' ref={refs.played}></div>
              </div>

            </div>

          </div>
          <div className='control'>
            <div className='left-control'>
              <Icon type="step-backward" className='icon-last' onClick={() => handleMusic('last')} />
              <span className={playing ? "icon-pause" : "icon-play"} onClick={() => handleMusic('play')}></span>
              {/*<Icon type={playing ? 'pause-circle' : 'play-circle'} className='icon-p' onClick={() => handleMusic('play')} theme="filled"></Icon>*/}
              <Icon type="step-forward" className='icon-last' onClick={() => handleMusic('next')} />
            </div>
            <div className='right-control'>
              <div className='volume-control-wrapper'
                onTouchMove={(e) => onMusicEvents(e, 'changeVolume')}
                onTouchStart={(e) => onMusicEvents(e, 'changeVolume')}
                onClick={(e) => onMusicEvents(e, 'changeVolume')}
                onMouseDown={(e) => onMusicEvents(e, 'mouseDown')}
                onMouseMove={(e) => onMusicEvents(e, 'slideVolume')}
                onMouseUp={(e) => onMusicEvents(e, 'mouseLeave')}
                onMouseLeave={(e) => onMusicEvents(e, 'mouseLeave')}
              >
                <div className='volume-control' ref={refs.totalVolume}>
                  <div className='volume-progress' ref={refs.volumeProgress}></div>
                </div>
              </div>
              <span className='icon-volume'></span>
            </div>
          </div>
        </div>
      </div>

    </AudioPlayer>
  )
}

export default forwardRef(WebAudio)
